/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */


/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* @(#) $Id: extern.h,v 1.24 2004/12/04 15:56:46 alex Exp $" */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef EXTERN_INCLUDED
#define EXTERN_INCLUDED

#ifndef EGETOPT
    extern int optind;
    extern char *optarg;
    extern int getopt();
#else
    extern int eoptind;
    extern char *eoptarg;
    extern int egetopt();
#define getopt egetopt
#define optarg eoptarg
#define optind eoptind
#endif

struct ifd_entry {
    unsigned short tag;
    unsigned short value_type;
    unsigned long count;
    unsigned long value;
};

struct tiff_image_summary {
    unsigned long offset;
    unsigned long size;
    int type;
    int compression;
};

struct maker_scheme {
    unsigned long scheme_type;
    unsigned long ifd_offset;   /* offset of IFD from start of note   */
    unsigned long fileoffset_base; /* entry offsets relative to this  */
    unsigned long id_length;    /* length of ID string, if present    */
    int num_entries;            /* from the IFD                       */
    unsigned short byteorder;   /* If there is a tiff header, or FUJI */
};

extern char *Progname;
extern long Header_offset;
extern unsigned long Max_undefined;
extern unsigned long Max_makerdump;
extern unsigned long Max_appdump;
extern unsigned long Print_options;

extern char *Make_name;
extern char *Model_name;
extern char *Software_name;

extern char *Use_Make_name;
extern char *Use_Model_name;
extern char *Use_Software_name;

/* =================== Function prototypes =========================  */
extern int process_options(int,char **);
extern void env_options();

extern unsigned long process_tiff_ifd(FILE *,unsigned short,unsigned long,
                                        unsigned long,unsigned long,
                                        int,int,int,int,int);
extern unsigned long process_exif_ifd(FILE *,unsigned short,unsigned long,
                                        unsigned long,unsigned long,
                                        int,int,int);
extern unsigned long process_gps_ifd(FILE *,unsigned short,unsigned long,
                                        unsigned long,unsigned long,
                                        int,int,int);
extern unsigned long process_subifd(FILE *,unsigned short,struct ifd_entry *,
                                        unsigned long,unsigned long,
                                        int,int,int,int);
extern unsigned long process_makernote(FILE *,unsigned short,struct ifd_entry *,
                                        unsigned long,unsigned long,
                                        int,int);
extern unsigned long process_app0(FILE *,unsigned long,unsigned short,int);
extern unsigned long process_app1(FILE *,unsigned long,unsigned short,int);
extern unsigned long process_app12(FILE *,unsigned long,unsigned short,int);
extern unsigned long process_appn(FILE *,unsigned long,unsigned short,int);
extern unsigned long process_jpeg_segments(FILE *,unsigned long,
                                            unsigned short,unsigned long,
                                            int,int);
extern int skip_past_newline(FILE *,unsigned long);
extern int skip_to_bracket(FILE *,unsigned long);
extern int putword(FILE *,int,unsigned long,int);

extern struct ifd_entry *read_entry(FILE *,unsigned short,unsigned long);
extern unsigned long read_ulong(FILE *,unsigned short,unsigned long);
extern unsigned long to_ulong(unsigned char *,unsigned short);
extern unsigned short read_ushort(FILE *,unsigned short,unsigned long);
extern unsigned short to_ushort(unsigned char *,unsigned short);
extern float read_float(FILE *,unsigned short,unsigned long);
extern float to_float(unsigned char *,unsigned short);
extern double read_double(FILE *,unsigned short,unsigned long);
extern double to_double(unsigned char *,unsigned short);
extern unsigned short read_ubyte(FILE *,unsigned long);
extern unsigned short read_imagemarker(FILE *,unsigned long);

extern unsigned char *read_bytes(FILE *,unsigned long,unsigned long);
extern char *read_appstring(FILE *,unsigned short,unsigned long);
extern char *read_string(FILE *,unsigned long,unsigned long);
extern void print_ubytes(FILE *,unsigned long,unsigned long);
extern void print_sbytes(FILE *,unsigned long,unsigned long);
extern void print_ushort(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_sshort(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_ulong(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_slong(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_urational(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_srational(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_float(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_double(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_user_comment(FILE *,unsigned long,unsigned long);
extern void print_cfapattern(FILE *,unsigned long,unsigned short,unsigned long);
extern void print_version();
extern void print_startvalue();
extern void print_endvalue();
extern void print_imagetype(unsigned short);

extern void print_tag_address(unsigned long,unsigned long,int,char *,int);
extern unsigned long print_entry(FILE *,unsigned short,int,int,int,int,
                                struct ifd_entry *,unsigned long,int);
extern int print_ascii(FILE *,unsigned long,unsigned long);
extern int print_header(unsigned short,unsigned long);
extern int value_type_size(unsigned short);
extern int is_a_long_offset(struct ifd_entry *);
extern void print_tagid(struct ifd_entry *,int,int);
extern void print_taginfo(struct ifd_entry *,int,int,int,int);
extern void print_value(struct ifd_entry *,char *);
extern void usage();
extern unsigned long print_offset_value(FILE *,unsigned short,
                                        struct ifd_entry *,
                                        unsigned long,int,int,int,int);
extern unsigned long sum_strip_bytecounts(FILE *,unsigned short,unsigned long,
                                            unsigned long,unsigned short);
extern unsigned long find_jpeg_eoi(FILE *,unsigned long,unsigned long);

extern int is_offset(struct ifd_entry *);
extern char *value_type_name(unsigned short);
extern char *comp_format(int);
extern char *get_type_name(int,unsigned long,unsigned long);
extern void print_tif_summary(struct tiff_image_summary *,
                    unsigned long,unsigned long,char *,int,int,
                    char *,int);

extern void putindent(int);
extern void extraindent(int);
extern void pushcolor(int);
extern void popcolor();
extern void printred(char *);
extern void why(FILE *);
extern unsigned long toggle(unsigned long,unsigned long);
extern void hexdump(FILE *,unsigned long,unsigned long,unsigned long,
                        int,int,int,int);
extern void clear_memory();

/* =============== Routines to interpret values ====================  */
extern void interpret_value(struct ifd_entry *);
extern void interpret_rational(FILE *,unsigned short byteorder,
                                unsigned short,unsigned long);

/* ====================== Maker functions ==========================  */

/* Generic                                                            */
extern char *tagname(unsigned short);
extern void print_makertagid(struct ifd_entry *,int,char *,int,int,int);
extern void print_makertaginfo(struct ifd_entry *,int,int,int);
extern char *interop_tagname(unsigned short);
extern char *gps_tagname(unsigned short);
extern char *maker_tagname(unsigned short,int,int);
extern int check_makernote(FILE*,unsigned long);
extern struct maker_scheme *find_makerscheme(FILE *,unsigned long,
                                                unsigned short,
                                                unsigned long,
                                                int,int);
extern unsigned long print_makerentry(FILE *,unsigned short,int,
                                struct ifd_entry *,unsigned long,
                                int,int,int);
extern void print_makervalue(struct ifd_entry *,int,int,char *);
extern unsigned long print_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern int maker_number(char *);
extern int model_number(int,char *,char *);
extern void find_maker_model(struct maker_scheme *,int,int);


extern void print_generic_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);

/* Agfa */
extern char *maker_agfa_tagname(unsigned short,int);
extern void print_agfa_makervalue(struct ifd_entry *,int,int,char *);
extern void print_agfa1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_agfa_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void agfa1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void agfa1_interpret_value(struct ifd_entry *);
extern void agfa1_interpret_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long);
extern int agfa_model_number(char *,char *);

/* Asahi/Pentax */
extern char *maker_asahi_tagname(unsigned short,int);
extern void print_asahi_makervalue(struct ifd_entry *,int,int,char *);
extern void print_asahi1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_asahi_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void asahi1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void asahi1_interpret_value(struct ifd_entry *);
extern int asahi_model_number(char *,char *);

/* Canon */
extern char *maker_canon_tagname(unsigned short,int);
extern void print_canon_makervalue(struct ifd_entry *,int,int,char *);
extern void print_canon1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_canon_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void canon1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void canon1_interpret_value(struct ifd_entry *);
extern int canon_model_number(char *,char *);

/* Casio */
extern char *maker_casio_tagname(unsigned short,int);

extern void print_casio_makervalue(struct ifd_entry *,int,int,char *);
extern void print_casio1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_casio_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void casio1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void casio1_interpret_value(struct ifd_entry *);
extern int casio_model_number(char *,char *);

/* Epson */
extern char *maker_epson_tagname(unsigned short,int);
extern void print_epson_makervalue(struct ifd_entry *,int,int,char *);
extern void print_epson1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_epson_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void epson1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void epson1_interpret_value(struct ifd_entry *);
extern void epson1_interpret_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long);
extern int epson_model_number(char *,char *);

/* Fujifilm */
extern char *maker_fujifilm_tagname(unsigned short,int);
extern void print_fujifilm_makervalue(struct ifd_entry *,int,int,char *);
extern void print_fujifilm1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_fujifilm_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void fujifilm1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void fujifilm1_interpret_value(struct ifd_entry *);
extern int fujifilm_model_number(char *,char *);

/* Leica */
extern char *maker_leica_tagname(unsigned short,int);
extern void print_leica_makervalue(struct ifd_entry *,int,int,char *);
extern void print_leica1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_leica_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void leica1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void leica1_interpret_value(struct ifd_entry *);
extern int leica_model_number(char *,char *);

/* Minolta */
extern char *maker_minolta_tagname(unsigned short,int);
extern void print_minolta_makervalue(struct ifd_entry *,int,int,char *);
extern void print_minolta1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_minolta_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void minolta1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void minolta1_interpret_value(struct ifd_entry *,int model);
extern void minolta1_interpret_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long);
extern int minolta_model_number(char *,char *);

/* Nikon */
extern char *maker_nikon_tagname(unsigned short,int);
extern char *maker_nikon1_tagname(unsigned short,int);
extern char *maker_nikon2_tagname(unsigned short,int);

extern void print_nikon_makervalue(struct ifd_entry *,int,int,char *);
extern void print_nikon1_makervalue(struct ifd_entry *,int,int,char *);
extern void print_nikon2_makervalue(struct ifd_entry *,int,int,char *);

extern void print_nikon_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void nikon1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void nikon2_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void nikon1_interpret_value(struct ifd_entry *);
extern void nikon2_interpret_value(struct ifd_entry *);
extern int nikon_model_number(char *,char *);

/* Olympus */
extern char *maker_olympus_tagname(unsigned short,int);
extern void print_olympus_makervalue(struct ifd_entry *,int,int,char *);
extern void print_olympus1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_olympus_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void olympus1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void olympus1_interpret_value(struct ifd_entry *);
extern void olympus1_interpret_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long);
extern int olympus_model_number(char *,char *);

/* Panasonic */
extern char *maker_panasonic_tagname(unsigned short,int);
extern void print_panasonic_makervalue(struct ifd_entry *,int,int,char *);
extern void print_panasonic1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_panasonic_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void panasonic1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void panasonic1_interpret_value(struct ifd_entry *);
extern void panasonic1_interpret_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long);
extern int panasonic_model_number(char *,char *);

/* Sanyo */
extern char *maker_sanyo_tagname(unsigned short,int);
extern void print_sanyo_makervalue(struct ifd_entry *,int,int,char *);
extern void print_sanyo1_makervalue(struct ifd_entry *,int,int,char *);

extern void print_sanyo_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void sanyo1_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long,
                                                char *prefix,
                                                int,int,int,int,int);
extern void sanyo1_interpret_value(struct ifd_entry *);
extern void sanyo1_interpret_offset_makervalue(FILE *,unsigned short,
                                                struct ifd_entry *,
                                                unsigned long);
extern int sanyo_model_number(char *,char *);

#endif
