/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: main.c,v 1.17 2003/04/21 00:57:43 alex Exp $";
#endif

/* Attempts to follow the structure of a Blurfle Interchange Format   */
/* image file containing all manner of TIFF and JPEG marker tags, and */
/* to report the structure and info content of the file. Makes no     */
/* attempt to interpret, use, or make fun of the actual image data.   */

/* This will typically read and report data in image files created    */
/* by programs whose authors have glanced through the TIFF6, JFIF,    */
/* EXIF, and TIFF/EP specs, and may be able to dig some incriminating */
/* evidence out of other files which use similar markers.             */

/* Image files are expected to start with either a JPEG SOI marker or */
/* a TIFF header.                                                     */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "defs.h"
#include "global.h"
#include "extern.h"
#include "misc.h"
#include "tags.h"

int
main(int argc,char **argv)
{
    char *file;
    FILE *inptr;
    int status = -1;
    unsigned long max_offset = 0UL;
    unsigned long ifd_offset = 0UL;
    unsigned long tag_offset = 0UL;
    unsigned short marker,tag;
    int opt;

    Progname = *argv;

    /* Gather options from EXIFPROBE_OPTIONS first                    */
    env_options();

    /* Now the command line                                           */
    argv += process_options(argc,argv);

    while(file = *argv++)
    {
        inptr = fopen(file,"r");
        if(inptr == FNULL)
        {
            fprintf(stderr,"%s: failed to open file %s\n",Progname,file);
            why(stderr);
            max_offset = 0L;
        }
        else
        {
            STOPCOLOR;  /* start in a known state, if using color     */
            printf("FILE = %s\n",file);    /* record the file name    */
            marker = read_imagemarker(inptr,HERE);
            print_imagetype(marker);

            if(PRINT_SECTION)
            {
                print_tag_address(SECTION,0L,0,"@",ADDRWIDTH);
                status = print_header(marker,SECTION);
            }
            else if((PRINT_SEGMENT) && (marker == JPEG_SOI))
            {
                print_tag_address(SEGMENT,0L,0,"@",ADDRWIDTH);
                status = print_header(marker,SEGMENT);
            }
            else
                status = print_header(marker,0UL);
            if(status == 0)
            {
                switch(marker)
                {
                    case TIFF_INTEL:
                    case TIFF_MOTOROLA:
                        ifd_offset = read_ulong(inptr,marker,HERE); 
                        if(PRINT_SECTION)
                            printf(" ifd offset = %#lx/%lu)\n",ifd_offset,ifd_offset);
                        max_offset = process_tiff_ifd(inptr,marker,
                                            ifd_offset,0L,0L,TIFF_IFD,
                                            0,-1,0,ADDRWIDTH);
                        break;
                    case JPEG_SOI:
                        tag_offset = ftell(inptr);
                        tag = read_ushort(inptr,TIFF_MOTOROLA,HERE);
                        while((status == 0) && (tag != 0))
                        {
                            /* There is so much we don't know...      */
                            switch(tag)
                            {
                                case JPEG_APP0: /* JFIF               */
                                    max_offset = process_app0(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                case JPEG_APP1: /* EXIF               */
                                    max_offset = process_app1(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                case JPEG_APP2: /* EXIF flashpix      */
                                                /* maybe some day     */
                                case JPEG_APP3: /*                    */
                                case JPEG_APP4: /*                    */
                                case JPEG_APP5: /*                    */
                                case JPEG_APP6: /*                    */
                                case JPEG_APP7: /*                    */
                                case JPEG_APP8: /*                    */
                                case JPEG_APP9: /*                    */
                                case JPEG_APP10:    /*                */
                                case JPEG_APP11:    /*                */
                                    max_offset = process_appn(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                case JPEG_APP12:    /*  Adobe         */
                                    max_offset = process_app12(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                case JPEG_APP13:    /*                */
                                    max_offset = process_appn(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                case JPEG_APP14:    /*                */
                                    max_offset = process_appn(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                case JPEG_APP15:    /*                */
                                    max_offset = process_appn(inptr,tag_offset,tag,SUBINDENT);
                                    break;
                                default:
                                    max_offset = process_jpeg_segments(inptr,tag_offset,tag,0L,0,ADDRWIDTH);
                                    status = -1;
                                    break;
                            }
                            tag_offset = max_offset;
                            if(!feof(inptr))
                                tag = read_ushort(inptr,TIFF_MOTOROLA,tag_offset);
                            else
                                tag = 0;    /* done                   */
                        }
                        if(max_offset > 0L)
                            status = 0;
                        fflush(stdout);
                        break;
                    default:
                        status = -1;
                        break;
                }
            }
            if(fseek(inptr,0L,2) == 0)
                print_tag_address(SECTION,ftell(inptr),0,"-",ADDRWIDTH);
            if(PRINT_SECTION)
                printf("END OF FILE\n\n");
        }
        clear_memory();     /* Make, Model, Software...               */
    }
    return(status);
}
