/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: maker_nikon.c,v 1.18 2004/12/06 17:48:33 alex Exp $";
#endif

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Nikon camera maker-specific routines.                              */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Much of the information coded here is due to TsuruZoh Tachibanaya  */
/* at:                                                                */
/* http://www.ba.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html   */

/* and Max Lyons (E990 data) at:                                      */
/* http://www.tawbaware.com/                                          */

/* Some information is taken from Eugene Crosser's "photopc" program: */
/* http://photopc.sourceforge.net/                                    */
/* (as modified by John Bowman: http://www.math.ualberta.ca/imaging/) */

/* The presence of JPEG thumbnails in TIFF format makernotes,         */
/* extension of (some) E990 tags to the E995 and later models, the    */
/* extra TIFF header in D100s, and the PrintIM data in some models,   */
/* is due to examination (by earlier versions of this program) of     */
/* images from my Nikon 990, and sample images obtained               */
/* from:                                                              */
/* http://www.imaging-resource.com/                                   */
/* and                                                                */
/* http://www.dpreview.com/                                           */

/* There are quite a few "unknown" tags for the E995 and 'D' cameras: */
/* in fact, those which are printed are *assumed* to be the same as   */
/* those listed by TsuruZoh Tachibanaya for the E990. This appears to */
/* be correct by examination of a few images from the above sources,  */
/* but should be confirmed by those who actually *have* those         */
/* cameras.                                                           */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "maker.h"

/* Find the identifying number assigned to known Nikon camera models. */
/* This number is used to dispatch print and interpret routines       */
/* apporopriate to the current image. At the moment, the "software"   */
/* string is not used.                                                */

int
nikon_model_number(char *model,char *software)
{
    int number = GENERIC;

    if(model)
    {
        switch(*model)
        {
            case 'E':
                /* This could be cleverer                         */
                if(strcmp(model,"E995") == 0)
                    number = NIKON_995;
                else if(strcmp(model,"E990") == 0)
                    number = NIKON_990;
                else if(strcmp(model,"E910") == 0)
                    number = NIKON_950;
                else if(strcmp(model,"E950") == 0)
                    number = NIKON_910;
                else if(strcmp(model,"E900") == 0)
                    number = NIKON_900;
                else if(strcmp(model,"E800") == 0)
                    number = NIKON_800;
                else if(strcmp(model,"E885") == 0)
                    number = NIKON_885;
                else if(strcmp(model,"E700") == 0)
                    number = NIKON_700;
                else if(strcmp(model,"E775") == 0)
                    number = NIKON_775;
                else if(strcmp(model,"E5000") == 0)
                    number = NIKON_5000;
                else if(strcmp(model,"E5700") == 0)
                    number = NIKON_5700;
                break;
            case 'N':
                if(strcmp(model,"NIKON D100 ") == 0) 
                    number = NIKON_D100;
                if(strncmp(model,"NIKON D1",8) == 0) 
                    number = NIKON_D1;
                break;
            default:
                number = GENERIC;
                break;
        }
    }
    return(number);
}


void
print_nikon_makervalue(struct ifd_entry *entry_ptr,int make, int model,
                    char *prefix)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case NIKON_700:
            case NIKON_800:
            case NIKON_900:
            case NIKON_910:
            case NIKON_950:
                print_nikon1_makervalue(entry_ptr,make,model,prefix);
                nikon1_interpret_value(entry_ptr);
                break;
            case NIKON_775:
            case NIKON_885:
            case NIKON_990:
            case NIKON_995:
            case NIKON_D1:
            case NIKON_D100:
            case NIKON_5000:
            case NIKON_5700:
            case GENERIC:       /* possibly unwarrented assumption        */
            default:
                print_nikon1_makervalue(entry_ptr,make,model,prefix);
                nikon2_interpret_value(entry_ptr);
                break;
        }
    }
}

void
print_nikon1_makervalue(struct ifd_entry *entry_ptr,int make, int model,
                    char *prefix)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(entry_ptr->tag) 
        {
            default:
                print_value(entry_ptr,PREFIX);
                break;
        }
    }
}

void
print_nikon_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case NIKON_700:
            case NIKON_800:
            case NIKON_900:
            case NIKON_910:
            case NIKON_950:
                nikon1_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,
                                            at_offset);
                break;
            case NIKON_775:
            case NIKON_885:
            case NIKON_990:
            case NIKON_995:
            case NIKON_D1:
            case NIKON_D100:
            case NIKON_5000:
            case NIKON_5700:
            default:            /* default to the new version             */
                nikon2_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,
                                            at_offset);
                break;
        }
    }
}

void
nikon1_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    unsigned long value_offset;
    unsigned long dumplength;
    unsigned short marker;
    char *nameoftag;

    if(entry_ptr)
    {
        nameoftag = maker_tagname(entry_ptr->tag,make,model); 
        value_offset = offset_ref + entry_ptr->value;

        /* Pick off UNDEFINED tag types. The rest can be handled by   */
        /* the generic routine.                                       */
        switch(entry_ptr->tag)
        {
            case 0x0100: /* JPEG Thumbnail in TIFF MakerNote          */
                if(at_offset)
                {
                    print_tag_address(VALUE_AT_OFFSET,value_offset,indent,prefix,addrwidth);
                    extraindent(SUBINDENT);
                    printf("Start of JPEG Thumbnail");
                }
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu",value_offset);
                }
                printf(" length %ld",entry_ptr->count);
                marker = read_imagemarker(inptr,value_offset);
                if(PRINT_SEGMENT)
                {
                    putchar('\n');
                    (void)process_jpeg_segments(inptr,value_offset,
                                                marker,entry_ptr->count,
                                                indent+(SUBINDENT * 2),addrwidth);
                    print_tag_address(SEGMENT,value_offset + entry_ptr->count - 1,
                                                indent,"-",addrwidth);
                    extraindent(SUBINDENT);
                    printf("End of JPEG Thumbnail");
                }
                /* make certain we're at the end unless 'inptr' bad   */
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            case 0x0e00: /* PrintIM (Epson Print Image matching)      */
            case 0x0f00: /* Data                                      */
                if(at_offset)
                {
                    print_tag_address(ENTRY,value_offset,indent,prefix,addrwidth);
                    print_makertagid(entry_ptr,19," : ",SUBINDENT,make,model);
                }
                else if(PRINT_OFFSET)
                        printf(" = ");
                printf("UNDEFINED, @%lu count %lu",value_offset, entry_ptr->count);
                if(Max_undefined == 0)
                    printred(" (not dumped, use -U)");
                else
                {
                    if((Max_undefined == DUMPALL)
                        || (Max_undefined > entry_ptr->count))
                            dumplength = entry_ptr->count;
                    else 
                        dumplength = Max_undefined;
                    putchar('\n');
                    hexdump(inptr,value_offset,entry_ptr->count,dumplength,12,
                                indent,SUBINDENT,addrwidth);
                }
                /* make certain we're at the end unless 'inptr' bad   */
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}

void
nikon2_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    unsigned long value_offset;
    unsigned long dumplength;
    unsigned short marker;
    char *nameoftag;

    if(entry_ptr)
    {
        nameoftag = maker_tagname(entry_ptr->tag,make,model); 
        value_offset = offset_ref + entry_ptr->value;
        switch(entry_ptr->tag)
        {
            case 0x0100: /* JPEG Thumbnail in TIFF MakerNote, for     */
                if(at_offset)
                {
                    print_tag_address(VALUE_AT_OFFSET,value_offset,indent,prefix,addrwidth);
                    extraindent(SUBINDENT);
                    printf("Start of JPEG Thumbnail");
                }
                if(!(PRINT_OFFSET))
                {
                    if(PRINT_BOTH_VALUE)
                        printf("@%#lx=%lu",value_offset,value_offset);
                    else if(PRINT_HEX_VALUE)
                        printf("@%#lx",value_offset);
                    else if(PRINT_DEC_VALUE)
                        printf("@%lu",value_offset);
                }
                printf(" length %ld",entry_ptr->count);
                marker = read_imagemarker(inptr,value_offset);
                if(PRINT_SEGMENT)
                {
                    putchar('\n');
                    (void)process_jpeg_segments(inptr,value_offset,
                                                marker,entry_ptr->count,
                                                indent+(SUBINDENT * 2),addrwidth);
                    print_tag_address(SEGMENT,value_offset + entry_ptr->count - 1,
                                                indent,"-",addrwidth);
                    extraindent(SUBINDENT);
                    printf("End of JPEG Thumbnail");
                }
                /* make certain we're at the end unless 'inptr' bad   */
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            case 0x0e00: /* PrintIM                                 */
            case 0x0010: /* Data                                      */
                if(at_offset)
                {
                    print_tag_address(ENTRY,value_offset,indent,prefix,addrwidth);
                    print_makertagid(entry_ptr,19," : ",SUBINDENT,make,model);
                }
                else if(PRINT_OFFSET)
                        printf(" = ");
                printf("UNDEFINED, @%lu count %lu",value_offset, entry_ptr->count);
                if(Max_undefined == 0)
                    printred(" (not dumped, use -U)");
                else
                {
                    if((Max_undefined == DUMPALL)
                        || (Max_undefined > entry_ptr->count))
                            dumplength = entry_ptr->count;
                    else 
                        dumplength = Max_undefined;
                    putchar('\n');
                    hexdump(inptr,value_offset,entry_ptr->count,dumplength,12,
                                indent,SUBINDENT,addrwidth);
                }
                /* make certain we're at the end          */
                fseek(inptr,value_offset + entry_ptr->count,0);
                break;
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}

char *
maker_nikon_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    /* Check specific models first                                    */
    switch(model)
    {
        case NIKON_950:
        case NIKON_910:
        case NIKON_900:
        case NIKON_800:
        case NIKON_700:
            tagname = maker_nikon1_tagname(tag,model);
            break;
        case NIKON_775:
        case NIKON_885:
        case NIKON_990:
        case NIKON_995:
        case NIKON_D1:
        case NIKON_D100:
        case NIKON_5000:
        case NIKON_5700:
        default:
            tagname = maker_nikon2_tagname(tag,model);
            break;
    }
    return(tagname);
}

/* Tags for the older cameras.                                        */

char *
maker_nikon1_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    switch(tag)
    {
        case 0x0003: tagname = "Quality"; break;
        case 0x0004: tagname = "ColorMode"; break;
        case 0x0005: tagname = "ImageAdjustment"; break;
        case 0x0006: tagname = "ISOSetting"; break;
        case 0x0007: tagname = "WhiteBalance"; break;
        case 0x0008: tagname = "Focus"; break;
        case 0x000a: tagname = "DigitalZoom"; break;
        case 0x000b: tagname = "LensConverter"; break;
        case 0x0100: tagname = "JPEG_Thumbnail"; break;
        case 0x0f00: tagname = "Data"; break;
            default: break;
    }
    return(tagname);
}

/* This is for the E990; some of the tags may be valid for the 995    */
/* and maybe 'D' cameras.                                             */
/* Tags which have been observed, but whose function is unknown are   */
/* commented out, so that the tag numbers will appear in place of the */
/* name. If you know what these are, send mail.                       */

/* The PrintIM tag is clear from hex/ascii dumps, although the format */
/* of the data is unknown...nobody's talking.  Ask Epson...           */

char *
maker_nikon2_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    switch(tag)
    {
        case 0x0001: tagname = "Version"; break;
        case 0x0002: tagname = "ISOSetting"; break;
        case 0x0003: tagname = "ColorMode"; break;
        case 0x0004: tagname = "Quality"; break;
        case 0x0005: tagname = "WhiteBalance"; break;
        case 0x0006: tagname = "ImageSharpening"; break;
        case 0x0007: tagname = "FocusMode"; break;
        case 0x0008: tagname = "FlashSetting"; break;
#if 0
        case 0x0009: break; /* don't know D1                          */
        case 0x000a: break; /* don't know E995                        */
        case 0x000b: break; /* don't know D1                          */
        case 0x000c: break; /* don't know D1                          */
        case 0x000d: break; /* don't know D1                          */
        case 0x000e: break; /* don't know D1                          */
#endif
        case 0x000f: tagname = "ISOSelection"; break;
        case 0x0010: tagname = "Data"; break;
#if 0
        case 0x0011: break; /* don't know E995                        */
#endif
        case 0x0080: tagname = "ImageAdjustment"; break;
#if 0
        case 0x0081: break; /* don't know D1                          */
#endif
        case 0x0082: tagname = "LensAdapter"; break;
#if 0
        case 0x0083: break; /* don't know D1                          */
        case 0x0084: break; /* don't know D1                          */
#endif
        case 0x0085: tagname = "ManualFocusDistance"; break;
        case 0x0086: tagname = "DigitalZoom"; break;
#if 0
        case 0x0087: break; /* don't know D1                          */
#endif
        case 0x0088: tagname = "AutoFocusPosition"; break;
        /* next 3: Johannes Tschebisch http://www.jojotsch.de         */
        case 0x008f: tagname = "SceneMode"; break;
        case 0x0094: tagname = "Saturation"; break;
        case 0x0095: tagname = "NoiseReduction"; break;

        case 0x0e00: tagname = "PrintIMData"; break;
        case 0x0100: tagname = "JPEG_Thumbnail"; break;
        default: break;
    }
    return(tagname);
}

void
nikon1_interpret_value(struct ifd_entry *entry_ptr)
{
    switch(entry_ptr->tag)
    {
        case 0x0003:    /* Quality */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1:
                    printf("VGA Basic - 640x480 (1/16)");
                    break;
                case 2:
                    printf("VGA Normal - 640x480 (1/8)");
                    break;
                case 3:
                    printf("VGA Fine - 640x480 (1/4)");
                    break;
                case 4:
                    printf("XGA Basic - 1024x768 (1/16)"); 
                    break;
                case 5:
                    printf("XGA Normal - 1024x768 (1/8)");
                    break;
                case 6:
                    printf("XGA Fine - 1024x768 (1/4)");
                    break;
                case 7:
                    printf("XGA Basic - 1024x768 (1/16)");
                    break;
                case 8:
                    printf("XGA Normal - 1024x768 (1/8)");
                    break;
                case 9:
                    printf("XGA Fine - 1024x768 (1/4)");
                    break;
                case 10:
                    printf("Basic - 1600x1200 (1/16)");
                    break;
                case 11:
                    printf("Normal - 1600x1200 (1/8)");
                    break;
                case 12:
                    printf("Fine - 1600x1200 (1/4)");
                    break;
                case 20:
                    printf("1600x1200 - Hi (1/1)");
                    break;
                default:
                    printred("undefined");
                    break;
            }
            print_endvalue();
            break;
        case 0x0004:    /* Color Mode */
            print_startvalue();
            if(entry_ptr->value == 1)
                printf("color");
            else if(entry_ptr->value == 2)
                printf("monochrome");
            else
                printred("undefined");
            print_endvalue();
            break;
        case 0x0005:    /* Image Adjustment */
            print_startvalue();
            /* The data from Crosser's "photopc" conflicts with that  */
            /* from Tachibanaya for contrast and brightness (they're  */
            /* swapped). These are Tachibanaya's values. Somebody please  */
            /* tell me which is right.                                */
            switch(entry_ptr->value)
            {
                case 0:
                    printf("Standard");
                    break;
                case 1:
                    printf("Brightness+?"); /* ###%%% Contrast?    */
                    break;
                case 2:
                    printf("Brightness-?");  /* ###%%% Contrast?   */
                    break;
                case 3:
                    printf("Contrast+?"); /* ###%%% Brightness?    */
                    break;
                case 4:
                    printf("Contrast-?"); /* ###%%% Brightness?    */
                    break;
                case 5:
                    /* Is this right?                                 */
                    printf("Auto?");
                    break;
                default:
                    printred("undefined");
                    break;
            }
            print_endvalue();
            break;
        case 0x0006:    /* CCD Sensitivity (ISO) */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0:
                    printf("80");
                    break;
                case 2:
                    printf("160");
                    break;
                case 4:
                    printf("320");
                    break;
                case 5:
                    printf("100");
                    break;
                default:
                    printred("undefined");
                    break;
            }
            print_endvalue();
            break;
        case 0x0007:    /* White Balance */
            print_startvalue();
            /* These values, taken from "photopc", conflict with      */
            /* values given by Tachibanaya                            */
            switch(entry_ptr->value)
            {
                case 0:
                    printf("Auto");
                    break;
                case 1:
                    printf("Preset");
                    break;
                case 2:
                    printf("Daylight");
                    break;
                case 3:
                    printf("Incandescent");
                    break;
                case 4:
                    printf("Fluorescent");
                    break;
                case 5:
                    printf("Cloudy");
                    break;
                case 6:
                    printf("Flash");
                    break;
                default:
                    printred("undefined");
                    break;
            }
            print_endvalue();
            break;
        case 0x000b:    /* Lens Converter */
            print_startvalue();
            if(entry_ptr->value == 0)
                printf("none");
            else if(entry_ptr->value == 1)
                printf("fisheye");
            else
                printred("undefined");
            print_endvalue();
            break;

        default:
            break;
    }
}

void 
nikon2_interpret_value(struct ifd_entry *entry_ptr)
{
    
    switch(entry_ptr->tag)
    {
        case 0x0002:    /* CCD Sensitivity (ISO) */
            print_startvalue();
            if(entry_ptr->value & 0xffff)
                printf("%u",(unsigned short)(entry_ptr->value & 0xffff));
            else if((entry_ptr->value >> 16) & 0xffff)
                printf("%u",(unsigned short)((entry_ptr->value >> 16) & 0xffff));
            else
                printf("Auto");
            print_endvalue();
            break;
        case 0x000b:
        case 0x000d:
        case 0x000e:
        case 0x0011:
        case 0x0083:
        case 0x0087:
        case 0x0089:
        case 0x008a:
        case 0x008b:
        case 0x0092:
        case 0x0094:
            print_startvalue();
            printred("undefined");
            print_endvalue();
            break;
        case 0x0088:    /* Auto Focus Position                    */
            print_startvalue();
            /* It appears that this byte describes autofocus      */
            /* position.                                          */
            /* It is not clear what the other bytes are for.      */
            /* Possibilities are:                                 */
            /*     metering lock to the focus area                */
            /*     auto vs manual focus area select or "off"      */
            /*     continuous AF vs single AF                     */
            switch((entry_ptr->value >> 8) & 0xff)
            {
                case 0:
                    printf("Center");
                    break;
                case 0x1:
                    printf("Top");
                    break;
                case 0x2:
                    printf("Bottom");
                    break;
                case 0x3:
                    printf("Left");
                    break;
                case 0x4:
                    printf("Right");
                    break;
                default:
                    printred("undefined");
                    break;
            }
            print_endvalue();
            break;
        default:
            break;
    }
}
