/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: misc.c,v 1.11 2003/05/14 21:53:26 alex Exp $";
#endif

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"

/* Return the size in bytes of a TIFF type                            */

int
value_type_size(unsigned short value_type)
{
    int size = 0;

    switch(value_type)
    {
        case BYTE:
        case ASCII:
        case SBYTE:
        case UNDEFINED:
            size = 1;
            break;
        case SHORT:
        case SSHORT:
            size = 2;
            break;
        case LONG:
        case SLONG:
        case FLOAT:
            size = 4;
            break;
        case RATIONAL:
        case SRATIONAL:
        case DOUBLE:
            size = 8;
            break;
        default:
            break;
    }
    return(size);
}

/* Return a string naming the TIFF type of a TIFF entry               */

char *
value_type_name(unsigned short value_type)
{
    char *name;

    switch(value_type)
    {
        case BYTE: name = "BYTE"; break;
        case ASCII: name = "ASCII"; break;
        case SBYTE: name = "SBYTE"; break;
        case UNDEFINED: name = "UNDEFINED"; break;
        case SHORT: name = "SHORT"; break;
        case SSHORT: name = "SSHORT"; break;
        case LONG: name = "LONG"; break;
        case SLONG: name = "SLONG"; break;
        case FLOAT: name = "FLOAT"; break;
        case RATIONAL: name = "RATIONAL"; break;
        case SRATIONAL: name = "SRATIONAL"; break;
        case DOUBLE: name = "DOUBLE"; break;
        default: name = "INVALID"; break;
    }
    return(name);
}

/* Produce an image format string corresponding to the compression    */
/* type found in a TIFF IFD. TIFF6/TIFFEP                             */
char *
comp_format(int compression)
{
    static char *name = (char *)0;

    /* This is all the values that I know about. Are there more?      */
    switch(compression)
    {
        /* will be 0 only if compression tag does not appear          */
        case 0: name = "JPEG compressed primary"; break; /* JPEG 10918-1 */
        case 1: name = "uncompressed"; break;   /* TIFF6/TIFF_EP      */
        case 2:
                name = "Modifed Huffman RLE compressed";    /* TIFF6  */
                break;
        case 3: name = "T4 fax encoded"; break; /* TIFF6              */
        case 4: name = "T6 fax encoded"; break; /* TIFF6              */
        case 5: name = "LZW compressed"; break; /* TIFF6              */
        case 6: name = "JPEG compressed thumbnail"; break; /* JPEG 10918-1 */
        case 7: name = "JPEG compressed"; break;    /* TIFF_EP        */
        case 32773: name = "Packbits RLE compressed"; break; /* TIFF6 */
        /* These are all from "libtiff -- tiff.h"                     */
        case 8: name="Adobe Deflate compressed"; break;
        case 32771: name="NeXT 2-bit RLE compressed"; break;
        case 32809: name="ThunderScan RLE compressed"; break;
        case 32895: name="IT8 CT compressed with padding"; break;
        case 32896: name="IT8 Linework compressed"; break;
        case 32897: name="IT8 Monochromecompressed"; break;
        case 32898: name="IT8 Binary line art compressed"; break;
        case 32908: name="Pixar companded 10bit LZW compressed"; break;
        case 32909: name="Pixar companded 11bit ZIP compressed"; break;
        case 32946: name="Deflate compressed"; break;
        case 32947: name="Kodak DCS encoded"; break;
        case 34661: name="ISO JBIG compressed"; break;
        case 34676: name="SGI Log Luminance RLE compressed"; break;
        case 34677: name="SGI Log 24-bit compressed"; break;
        default: name = "UNKNOWN compression format"; break;
    }
    return(name);
}

/* Return a string describing the type (primary or thumbnail) of the  */
/* image represented by 'imagetype'. If that type is not known from a */
/* TIFF/EP NewSubFileType tag, assume that the image described by     */
/* image1_size is the current image, from a pair of images described  */
/* by two consecutive IFDS. The larger of the two is primary. If      */
/* image2_size is 0, there is only one IFD, and the image must be     */
/* primary.                                                           */

char *
get_type_name(int imagetype,unsigned long image1_size,unsigned long image2_size)
{
    char *type_of_image = UNKNOWN;

    if(imagetype == THUMBNAIL_TYPE)
        type_of_image = THUMBNAIL;
    else if(imagetype == PRIMARY_TYPE)
        type_of_image = PRIMARY;
    else if(imagetype == PAGE_TYPE)
        type_of_image = PAGE;
    else if(imagetype == MASK_TYPE)
        type_of_image = MASK;
    else    /* not TIFF/EP                                */
    {
        type_of_image = UNKNOWN;
        if(image2_size == 0L)
            type_of_image = PRIMARY;
        else
        {
            if(image1_size > image2_size)
                type_of_image = PRIMARY;
            else
                type_of_image = THUMBNAIL;
        }
    }
    return(type_of_image);
}

/* summmarize the location and size of an image in a TIFF format      */
/* file, from information recorded in the argument tif_image_summary  */
/* structure, with the aid of "other_size" if necessary to determine  */
/* whether the image is primary or thumbnail.                         */

void
print_tif_summary(struct tiff_image_summary *tif_img,
                    unsigned long other_size,
                    unsigned long tiffheader_offset,char *ifdname,
                    int ifdnum,int indent,char *prefix,int addrwidth)
{
    char *image_format,*type_of_image;      /* For TIFF format        */

    image_format = comp_format(tif_img->compression);
    type_of_image = get_type_name(tif_img->type,tif_img->size,other_size);

    print_tag_address(SECTION,tif_img->offset + tiffheader_offset,
                        indent,"@",addrwidth);
    printf("Start of %s %s data, length %lu (%s %d)",
            image_format,type_of_image,tif_img->size,ifdname,ifdnum);
    if((PRINT_ADDRESS) == 0)   /* print_tag_address didn't do it     */
        printf(" at offset %#lx/%lu\n",tif_img->offset + tiffheader_offset,
            tif_img->offset + tiffheader_offset);
    else
        putchar('\n');
    print_tag_address(SECTION,tif_img->offset +
                        tif_img->size +
                        tiffheader_offset,indent,"-",
                        addrwidth);
    printf("  End of %s %s data",image_format,type_of_image);
    if((PRINT_ADDRESS) == 0)   /* print_tag_address didn't do it     */
        printf(" at offset %#lx/%lu\n",tif_img->offset + tif_img->size +
                        tiffheader_offset,tif_img->offset +
                        tif_img->size + tiffheader_offset);
    else
        putchar('\n');
}

/* Sum the strip or type bytecounts from a TIFF header to yield the   */
/* size of the image data. The bytecounts may be short or long, so    */
/* the value_type must be checked.                                    */

unsigned long
sum_strip_bytecounts(FILE *inptr,unsigned short byteorder,unsigned long offset,
    unsigned long count,unsigned short value_type)
{
    int i;
    unsigned long sum = 0L;

    if(count)
    {
        /* Some use SHORT, some use LONG. If anyone uses some other   */
        /* type, just ignore 'em.                                     */
        if(value_type == SHORT)
        {
            sum = read_ushort(inptr,byteorder,offset);
            for(i = 1; i < count; ++i)
                sum += read_ushort(inptr,byteorder,HERE);
        }
        else if(value_type == LONG)
        {
            sum = read_ulong(inptr,byteorder,offset);
            for(i = 1; i < count; ++i)
                sum += read_ulong(inptr,byteorder,HERE);
        }
    }
    return(sum);
}


void
why(FILE *outptr)
{
    fprintf(outptr,"\t because: %s\n",strerror(errno));
    errno = 0;
}

unsigned long
toggle(unsigned long options,unsigned long bits)
{
    if(options & bits)
        options &= ~bits;
    else
        options |= bits;
    return(options);
}


/* Clean up memory used to save Make_name, etc.                       */
void
clear_memory()
{
    if(Make_name)
        free(Make_name);
    if(Model_name)
        free(Model_name);
    if(Software_name)
        free(Software_name);
    Make_name = Model_name = Software_name = (char *)0;
}
